package com.tzld.piaoquan.longarticle.service.local.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.longarticle.dao.mapper.longarticle.LongArticlesTextMapper;
import com.tzld.piaoquan.longarticle.model.po.longarticle.LongArticlesText;
import com.tzld.piaoquan.longarticle.model.po.longarticle.LongArticlesTextExample;
import com.tzld.piaoquan.longarticle.service.local.KimiService;
import com.tzld.piaoquan.longarticle.utils.other.OpenAIUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

@Slf4j
@Service
public class KimiServiceImpl implements KimiService {

    private static final Integer SAFE_SCORE = 7;

    @Autowired
    private LongArticlesTextMapper longArticlesTextMapper;

    @Override
    public LongArticlesText getKimiText(String contentId) {
        LongArticlesTextExample example = new LongArticlesTextExample();
        example.createCriteria().andContentIdEqualTo(contentId);
        List<LongArticlesText> longArticlesTexts = longArticlesTextMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(longArticlesTexts)) {
            return null;
        }
        return longArticlesTexts.get(0);
    }

    public boolean updateKimiContent(String contentId) {
        LongArticlesTextExample example = new LongArticlesTextExample();
        example.createCriteria().andContentIdEqualTo(contentId);
        List<LongArticlesText> longArticlesTexts = longArticlesTextMapper.selectByExampleWithBLOBs(example);
        log.info("updateKimiContent contentId={} longArticlesTexts={}", contentId, longArticlesTexts);
        if (CollectionUtils.isEmpty(longArticlesTexts)) {
            return false;
        }
        LongArticlesText longArticlesText = longArticlesTexts.get(0);
        if (longArticlesText.getKimiStatus() == 1) {
            return true;
        }
        String kimiTitle = OpenAIUtils.getKimiTitle(longArticlesText.getArticleTitle());
        if (StringUtils.isEmpty(kimiTitle)) {
            return false;
        }
        Integer score = OpenAIUtils.getKimiTitleSafeScore(kimiTitle);
        if (score == null || score > SAFE_SCORE) {
            kimiTitle = OpenAIUtils.makeKimiTitleSafer(longArticlesText.getArticleTitle());
        }
        kimiTitle = kimiTitle.replace("'", "").replace("\"", "").replace("\\", "");
        if (StringUtils.isEmpty(kimiTitle)) {
            return false;
        }
        JSONObject kimiInfo = OpenAIUtils.kimiMining(longArticlesText.getArticleText());
        if (kimiInfo == null) {
            return false;
        }
        String kimiSummary = kimiInfo.getString("content_title").replace("'", "").replace("\"", "");
        String kimiKeys = kimiInfo.getJSONArray("content_keys").toJSONString();
        log.info("updateKimiContent contentId={} kimiTitle={} kimiSummary={} kimiKeys={}", contentId, kimiTitle, kimiSummary, kimiKeys);
        if (StringUtils.isNotEmpty(kimiTitle) && StringUtils.isNotEmpty(kimiSummary) && StringUtils.isNotEmpty(kimiKeys)) {
            longArticlesText.setKimiTitle(kimiTitle);
            longArticlesText.setKimiSummary(kimiSummary);
            longArticlesText.setKimiKeys(kimiKeys);
            longArticlesText.setKimiStatus(1);
            longArticlesTextMapper.updateByPrimaryKeySelective(longArticlesText);
            return true;
        }
        return false;
    }

    public void updateKimiContentStatus(String contentId, Integer status) {
        LongArticlesTextExample example = new LongArticlesTextExample();
        example.createCriteria().andContentIdEqualTo(contentId);
        List<LongArticlesText> longArticlesTexts = longArticlesTextMapper.selectByExampleWithBLOBs(example);
        if (!CollectionUtils.isEmpty(longArticlesTexts)) {
            LongArticlesText longArticlesText = longArticlesTexts.get(0);
            longArticlesText.setKimiStatus(status);
            longArticlesTextMapper.updateByPrimaryKeySelective(longArticlesText);
        }
    }
}