package com.tzld.piaoquan.longarticle.utils.other;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.longarticle.common.constants.CrawlerConstant;
import com.tzld.piaoquan.longarticle.model.po.longarticle.CrawlerVideo;
import lombok.extern.slf4j.Slf4j;


import java.io.IOException;
import java.net.*;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Slf4j
public class HkspSearch {


    public static CrawlerVideo baiduVideoProduce(JSONObject jsonObject) {
        CrawlerVideo crawlerVideo = new CrawlerVideo();
        crawlerVideo.setPlatform(jsonObject.getString("platform"));
        crawlerVideo.setVideoTitle(jsonObject.getString("title"));
        crawlerVideo.setOutVideoId(jsonObject.getString("id"));
        if (jsonObject.getLong("publish_time") != null) {
            crawlerVideo.setPublishTime(new Date(jsonObject.getLong("publish_time") * 1000));
        }
        crawlerVideo.setVideoUrl(jsonObject.getString("playurl"));
        crawlerVideo.setCoverUrl(jsonObject.getString("poster"));
        crawlerVideo.setLikeCount(jsonObject.getInteger("like") == null ? 0 : jsonObject.getInteger("like"));
        crawlerVideo.setPlayCount(jsonObject.getInteger("playcnt"));
        crawlerVideo.setDuration(jsonObject.getInteger("duration"));
        return crawlerVideo;
    }

    public static void main(String[] args) {
        // 这行代码是身份验证的关键配置，不然身份验证不起作用
        System.setProperty("jdk.http.auth.tunneling.disabledSchemes", "");
        // 身份验证
        Authenticator.setDefault(
                new Authenticator() {
                    public PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(
                                "t11983523373311", "mtuhdr2z".toCharArray());
                    }
                }
        );
        List<String> sensitiveWords = new ArrayList<>();
        List<JSONObject> list = hkspSearch("代际沟通,家庭和谐,老年人适应", sensitiveWords, "");
        for (JSONObject jsonObject : list) {
            CrawlerVideo crawlerVideo = baiduVideoProduce(jsonObject);
            System.out.println(jsonObject);
            System.out.println(crawlerVideo);
        }
    }

    public static List<JSONObject> hkspSearch(String key, List<String> sensitiveWords, String traceId) {
        try {
            long timestampMilliseconds = System.currentTimeMillis();
            String url = "https://haokan.baidu.com/haokan/ui-search/pc/search/video";
            String encodedKey = URLEncoder.encode(key, "UTF-8");
            String strings = String.format("%d_%s_%d_%d_%d", 1, encodedKey, 10, timestampMilliseconds, 1);
            String sign = md5(strings);

            JSONObject params = new JSONObject();
            params.put("pn", 1);
            params.put("rn", 10);
            params.put("type", "video");
            params.put("query", key);
            params.put("sign", sign);
            params.put("version", 1);
            params.put("timestamp", timestampMilliseconds);

            String base64String = Base64.getEncoder().encodeToString(UUID.randomUUID().toString().getBytes());
            HttpResponse response = HttpRequest.get(url)
                    .header("authority", "haokan.baidu.com")
                    .header("accept", "*/*")
                    .header("accept-language", "zh,en;q=0.9,zh-CN;q=0.8")
                    .header("cookie", "BIDUPSID=" + base64String)
                    .header("user-agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36")
                    .header("x-requested-with", "xmlhttprequest")
                    .timeout(120000) // 设置超时时间
                    .setProxy(new Proxy(Proxy.Type.HTTP, new InetSocketAddress("l901.kdltps.com", 15818)))
                    .form(params)
                    .execute();

            List<JSONObject> resultList = new ArrayList<>();
            System.out.println(response.body());
            JSONObject jsonResponse = JSONObject.parseObject(response.body());
            JSONArray dataList = jsonResponse.getJSONObject("data").getJSONArray("list");

            for (int i = 0; i < dataList.size(); i++) {
                JSONObject data = dataList.getJSONObject(i);
                try {
                    String videoId = data.getString("vid");
                    String title = data.getString("title");
                    int duration = parseDuration(data.getString("duration"));

                    if (sensitiveFlag(sensitiveWords, title) && duration <= CrawlerConstant.MAX_VIDEO_DURATION) {
                        JSONObject res = getVideoDetail(videoId);
                        if (res != null) {
                            resultList.add(res);
                        }
                    }
                } catch (Exception e) {
                    // 处理异常
                }
            }
            return resultList;
        } catch (Exception e) {
            return new ArrayList<>();
        }
    }

    private static int parseDuration(String duration) {
        String[] parts = duration.split(":");
        return Integer.parseInt(parts[0]) * 60 + Integer.parseInt(parts[1]);
    }

    private static String md5(String input) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] messageDigest = md.digest(input.getBytes());
            StringBuilder sb = new StringBuilder();
            for (byte b : messageDigest) {
                sb.append(String.format("%02x", b));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            log.error("HkspSearch md5 error", e);
        }
        return "";
    }

    private static boolean sensitiveFlag(List<String> sensitiveWords, String title) {
        // 实现敏感词检查逻辑
        for (String word : sensitiveWords) {
            if (title.contains(word)) {
                return false;
            }
        }
        return true;
    }

    public static JSONObject getVideoDetail(String videoId) {
        String url = "https://haokan.baidu.com/v";
        JSONObject params = new JSONObject();
        params.put("vid", videoId);
        params.put("_format", "json");

        String base64String = Base64.getEncoder().encodeToString(UUID.randomUUID().toString().getBytes());
        HttpResponse response = HttpRequest.get(url)
                .header("Accept", "*/*")
                .header("cookie", "BIDUPSID=" + base64String)
                .header("Accept-Language", "en,zh;q=0.9,zh-CN;q=0.8")
                .header("Cache-Control", "no-cache")
                .header("Connection", "keep-alive")
                .header("Content-Type", "application/x-www-form-urlencoded")
                .header("Referer", "https://haokan.baidu.com")
                .header("User-Agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36")
                .timeout(120000) // 设置超时时间
                .form(params)
                .setProxy(new Proxy(Proxy.Type.HTTP, new InetSocketAddress("l901.kdltps.com", 15818)))
                .execute();
        return JSONObject.parseObject(response.body()).getJSONObject("data").getJSONObject("apiData").getJSONObject("curVideoMeta");
    }

}